"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_fetch_1 = require("node-fetch");
const FormData = require("form-data");
const api_client_1 = require("../api-client");
jest.mock('node-fetch', () => ({
    default: jest.fn(),
}));
function mockFetchResponse(response) {
    node_fetch_1.default.mockResolvedValue(response);
}
describe('ApiClient', () => {
    const testToken = 'test-token';
    const testDomain = 'test-domain.com';
    const testOrg = 'test-org';
    const testProject = 'test-project';
    describe('getDefaultBranch()', () => {
        let apiClient;
        beforeEach(() => {
            apiClient = new api_client_1.ReuniteApiClient(testDomain, testToken);
        });
        it('should get default project branch', () => __awaiter(void 0, void 0, void 0, function* () {
            mockFetchResponse({
                ok: true,
                json: jest.fn().mockResolvedValue({
                    branchName: 'test-branch',
                }),
            });
            const result = yield apiClient.remotes.getDefaultBranch(testOrg, testProject);
            expect(node_fetch_1.default).toHaveBeenCalledWith(`${testDomain}/api/orgs/${testOrg}/projects/${testProject}/source`, {
                method: 'GET',
                headers: {
                    'Content-Type': 'application/json',
                    Authorization: `Bearer ${testToken}`,
                },
                signal: expect.any(Object),
            });
            expect(result).toEqual('test-branch');
        }));
        it('should throw parsed error if response is not ok', () => __awaiter(void 0, void 0, void 0, function* () {
            mockFetchResponse({
                ok: false,
                json: jest.fn().mockResolvedValue({
                    type: 'about:blank',
                    title: 'Project source not found',
                    status: 404,
                    detail: 'Not Found',
                    object: 'problem',
                }),
            });
            yield expect(apiClient.remotes.getDefaultBranch(testOrg, testProject)).rejects.toThrow(new Error('Failed to fetch default branch: Project source not found'));
        }));
        it('should throw statusText error if response is not ok', () => __awaiter(void 0, void 0, void 0, function* () {
            mockFetchResponse({
                ok: false,
                statusText: 'Not found',
                json: jest.fn().mockResolvedValue({
                    unknownField: 'unknown-error',
                }),
            });
            yield expect(apiClient.remotes.getDefaultBranch(testOrg, testProject)).rejects.toThrow(new Error('Failed to fetch default branch: Not found'));
        }));
    });
    describe('upsert()', () => {
        const remotePayload = {
            mountBranchName: 'remote-mount-branch-name',
            mountPath: 'remote-mount-path',
        };
        let apiClient;
        beforeEach(() => {
            apiClient = new api_client_1.ReuniteApiClient(testDomain, testToken);
        });
        it('should upsert remote', () => __awaiter(void 0, void 0, void 0, function* () {
            const responseMock = {
                id: 'remote-id',
                type: 'CICD',
                mountPath: 'remote-mount-path',
                mountBranchName: 'remote-mount-branch-name',
                organizationId: testOrg,
                projectId: testProject,
            };
            mockFetchResponse({
                ok: true,
                json: jest.fn().mockResolvedValue(responseMock),
            });
            const result = yield apiClient.remotes.upsert(testOrg, testProject, remotePayload);
            expect(node_fetch_1.default).toHaveBeenCalledWith(`${testDomain}/api/orgs/${testOrg}/projects/${testProject}/remotes`, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                    Authorization: `Bearer ${testToken}`,
                },
                body: JSON.stringify({
                    mountPath: remotePayload.mountPath,
                    mountBranchName: remotePayload.mountBranchName,
                    type: 'CICD',
                    autoMerge: true,
                }),
                signal: expect.any(Object),
                agent: undefined,
            });
            expect(result).toEqual(responseMock);
        }));
        it('should throw parsed error if response is not ok', () => __awaiter(void 0, void 0, void 0, function* () {
            mockFetchResponse({
                ok: false,
                json: jest.fn().mockResolvedValue({
                    type: 'about:blank',
                    title: 'Not allowed to mount remote outside of project content path: /docs',
                    status: 403,
                    detail: 'Forbidden',
                    object: 'problem',
                }),
            });
            yield expect(apiClient.remotes.upsert(testOrg, testProject, remotePayload)).rejects.toThrow(new Error('Failed to upsert remote: Not allowed to mount remote outside of project content path: /docs'));
        }));
        it('should throw statusText error if response is not ok', () => __awaiter(void 0, void 0, void 0, function* () {
            mockFetchResponse({
                ok: false,
                statusText: 'Not found',
                json: jest.fn().mockResolvedValue({
                    unknownField: 'unknown-error',
                }),
            });
            yield expect(apiClient.remotes.upsert(testOrg, testProject, remotePayload)).rejects.toThrow(new Error('Failed to upsert remote: Not found'));
        }));
    });
    describe('push()', () => {
        const testRemoteId = 'test-remote-id';
        const pushPayload = {
            remoteId: testRemoteId,
            commit: {
                message: 'test-message',
                author: {
                    name: 'test-name',
                    email: 'test-email',
                },
                branchName: 'test-branch-name',
            },
        };
        const filesMock = [{ path: 'some-file.yaml', stream: Buffer.from('fefef') }];
        const responseMock = {
            branchName: 'rem/cicd/rem_01he7sr6ys2agb7w0g9t7978fn-main',
            hasChanges: true,
            files: [
                {
                    type: 'file',
                    name: 'some-file.yaml',
                    path: 'docs/remotes/some-file.yaml',
                    lastModified: 1698925132394.2993,
                    mimeType: 'text/yaml',
                },
            ],
            commitSha: 'bb23a2f8e012ac0b7b9961b57fb40d8686b21b43',
            outdated: false,
        };
        let apiClient;
        beforeEach(() => {
            apiClient = new api_client_1.ReuniteApiClient(testDomain, testToken);
        });
        it('should push to remote', () => __awaiter(void 0, void 0, void 0, function* () {
            let passedFormData = new FormData();
            node_fetch_1.default.mockImplementationOnce((_, options) => __awaiter(void 0, void 0, void 0, function* () {
                passedFormData = options.body;
                return {
                    ok: true,
                    json: jest.fn().mockResolvedValue(responseMock),
                };
            }));
            const formData = new FormData();
            formData.append('remoteId', testRemoteId);
            formData.append('commit[message]', pushPayload.commit.message);
            formData.append('commit[author][name]', pushPayload.commit.author.name);
            formData.append('commit[author][email]', pushPayload.commit.author.email);
            formData.append('commit[branchName]', pushPayload.commit.branchName);
            formData.append('files[some-file.yaml]', filesMock[0].stream);
            const result = yield apiClient.remotes.push(testOrg, testProject, pushPayload, filesMock);
            expect(node_fetch_1.default).toHaveBeenCalledWith(`${testDomain}/api/orgs/${testOrg}/projects/${testProject}/pushes`, expect.objectContaining({
                method: 'POST',
                headers: {
                    Authorization: `Bearer ${testToken}`,
                },
            }));
            expect(JSON.stringify(passedFormData).replace(new RegExp(passedFormData.getBoundary(), 'g'), '')).toEqual(JSON.stringify(formData).replace(new RegExp(formData.getBoundary(), 'g'), ''));
            expect(result).toEqual(responseMock);
        }));
        it('should throw parsed error if response is not ok', () => __awaiter(void 0, void 0, void 0, function* () {
            mockFetchResponse({
                ok: false,
                json: jest.fn().mockResolvedValue({
                    type: 'about:blank',
                    title: 'Cannot push to remote',
                    status: 403,
                    detail: 'Forbidden',
                    object: 'problem',
                }),
            });
            yield expect(apiClient.remotes.push(testOrg, testProject, pushPayload, filesMock)).rejects.toThrow(new Error('Failed to push: Cannot push to remote'));
        }));
        it('should throw statusText error if response is not ok', () => __awaiter(void 0, void 0, void 0, function* () {
            mockFetchResponse({
                ok: false,
                statusText: 'Not found',
                json: jest.fn().mockResolvedValue({
                    unknownField: 'unknown-error',
                }),
            });
            yield expect(apiClient.remotes.push(testOrg, testProject, pushPayload, filesMock)).rejects.toThrow(new Error('Failed to push: Not found'));
        }));
    });
});
