"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const push_1 = require("../push");
const api_1 = require("../../api");
const remotes = {
    push: jest.fn(),
    upsert: jest.fn(),
    getDefaultBranch: jest.fn(),
};
jest.mock('@redocly/openapi-core', () => ({
    slash: jest.fn().mockImplementation((p) => p),
}));
jest.mock('../../api', () => (Object.assign(Object.assign({}, jest.requireActual('../../api')), { ReuniteApiClient: jest.fn().mockImplementation(function (...args) {
        this.remotes = remotes;
    }) })));
describe('handlePush()', () => {
    let pathResolveSpy;
    let pathRelativeSpy;
    let pathDirnameSpy;
    let fsStatSyncSpy;
    let fsReaddirSyncSpy;
    beforeEach(() => {
        remotes.getDefaultBranch.mockResolvedValueOnce('test-default-branch');
        remotes.upsert.mockResolvedValueOnce({ id: 'test-remote-id', mountPath: 'test-mount-path' });
        remotes.push.mockResolvedValueOnce({ branchName: 'uploaded-to-branch', id: 'test-id' });
        jest.spyOn(fs, 'createReadStream').mockReturnValue('stream');
        pathResolveSpy = jest.spyOn(path, 'resolve');
        pathRelativeSpy = jest.spyOn(path, 'relative');
        pathDirnameSpy = jest.spyOn(path, 'dirname');
        fsStatSyncSpy = jest.spyOn(fs, 'statSync');
        fsReaddirSyncSpy = jest.spyOn(fs, 'readdirSync');
    });
    afterEach(() => {
        pathResolveSpy.mockRestore();
        pathRelativeSpy.mockRestore();
        pathDirnameSpy.mockRestore();
        fsStatSyncSpy.mockRestore();
        fsReaddirSyncSpy.mockRestore();
    });
    it('should upload files', () => __awaiter(void 0, void 0, void 0, function* () {
        const mockConfig = { apis: {} };
        process.env.REDOCLY_AUTHORIZATION = 'test-api-key';
        fsStatSyncSpy.mockReturnValueOnce({
            isDirectory() {
                return false;
            },
        });
        pathResolveSpy.mockImplementationOnce((p) => p);
        pathRelativeSpy.mockImplementationOnce((_, p) => p);
        pathDirnameSpy.mockImplementation((_) => '.');
        yield (0, push_1.handlePush)({
            domain: 'test-domain',
            'mount-path': 'test-mount-path',
            organization: 'test-org',
            project: 'test-project',
            branch: 'test-branch',
            namespace: 'test-namespace',
            repository: 'test-repository',
            'commit-sha': 'test-commit-sha',
            'commit-url': 'test-commit-url',
            'default-branch': 'test-branch',
            'created-at': 'test-created-at',
            author: 'TestAuthor <test-author@mail.com>',
            message: 'Test message',
            files: ['test-file'],
            'max-execution-time': 10,
        }, mockConfig);
        expect(remotes.getDefaultBranch).toHaveBeenCalledWith('test-org', 'test-project');
        expect(remotes.upsert).toHaveBeenCalledWith('test-org', 'test-project', {
            mountBranchName: 'test-default-branch',
            mountPath: 'test-mount-path',
        });
        expect(remotes.push).toHaveBeenCalledWith('test-org', 'test-project', {
            isMainBranch: true,
            remoteId: 'test-remote-id',
            commit: {
                message: 'Test message',
                branchName: 'test-branch',
                createdAt: 'test-created-at',
                namespace: 'test-namespace',
                repository: 'test-repository',
                sha: 'test-commit-sha',
                url: 'test-commit-url',
                author: {
                    name: 'TestAuthor',
                    email: 'test-author@mail.com',
                },
            },
        }, [
            {
                path: 'test-file',
                stream: 'stream',
            },
        ]);
    }));
    it('should return push id', () => __awaiter(void 0, void 0, void 0, function* () {
        const mockConfig = { apis: {} };
        process.env.REDOCLY_AUTHORIZATION = 'test-api-key';
        fsStatSyncSpy.mockReturnValueOnce({
            isDirectory() {
                return false;
            },
        });
        pathResolveSpy.mockImplementationOnce((p) => p);
        pathRelativeSpy.mockImplementationOnce((_, p) => p);
        pathDirnameSpy.mockImplementation((_) => '.');
        const result = yield (0, push_1.handlePush)({
            domain: 'test-domain',
            'mount-path': 'test-mount-path',
            organization: 'test-org',
            project: 'test-project',
            branch: 'test-branch',
            namespace: 'test-namespace',
            repository: 'test-repository',
            'commit-sha': 'test-commit-sha',
            'commit-url': 'test-commit-url',
            'default-branch': 'test-branch',
            'created-at': 'test-created-at',
            author: 'TestAuthor <test-author@mail.com>',
            message: 'Test message',
            files: ['test-file'],
            'max-execution-time': 10,
        }, mockConfig);
        expect(result).toEqual({ pushId: 'test-id' });
    }));
    it('should collect files from directory and preserve file structure', () => __awaiter(void 0, void 0, void 0, function* () {
        const mockConfig = { apis: {} };
        process.env.REDOCLY_AUTHORIZATION = 'test-api-key';
        /*
          ├── app
          │   ├── index.html
          ├── openapi.yaml
          └── some-ref.yaml
        */
        fsStatSyncSpy.mockImplementation((filePath) => ({
            isDirectory() {
                return filePath === 'test-folder' || filePath === 'test-folder/app';
            },
        }));
        fsReaddirSyncSpy.mockImplementation((dirPath) => {
            if (dirPath === 'test-folder') {
                return ['app', 'another-ref.yaml', 'openapi.yaml'];
            }
            if (dirPath === 'test-folder/app') {
                return ['index.html'];
            }
            throw new Error('Not a directory');
        });
        yield (0, push_1.handlePush)({
            domain: 'test-domain',
            'mount-path': 'test-mount-path',
            organization: 'test-org',
            project: 'test-project',
            branch: 'test-branch',
            author: 'TestAuthor <test-author@mail.com>',
            message: 'Test message',
            'default-branch': 'main',
            files: ['test-folder'],
            'max-execution-time': 10,
        }, mockConfig);
        expect(remotes.push).toHaveBeenCalledWith(expect.anything(), expect.anything(), expect.anything(), [
            {
                path: 'app/index.html',
                stream: 'stream',
            },
            {
                path: 'another-ref.yaml',
                stream: 'stream',
            },
            {
                path: 'openapi.yaml',
                stream: 'stream',
            },
        ]);
    }));
    it('should not upload files if no files passed', () => __awaiter(void 0, void 0, void 0, function* () {
        const mockConfig = { apis: {} };
        process.env.REDOCLY_AUTHORIZATION = 'test-api-key';
        yield (0, push_1.handlePush)({
            domain: 'test-domain',
            'mount-path': 'test-mount-path',
            organization: 'test-org',
            project: 'test-project',
            branch: 'test-branch',
            author: 'TestAuthor <test-author@mail.com>',
            message: 'Test message',
            'default-branch': 'main',
            files: [],
            'max-execution-time': 10,
        }, mockConfig);
        expect(remotes.getDefaultBranch).not.toHaveBeenCalled();
        expect(remotes.upsert).not.toHaveBeenCalled();
        expect(remotes.push).not.toHaveBeenCalled();
    }));
    it('should get organization from config if not passed', () => __awaiter(void 0, void 0, void 0, function* () {
        const mockConfig = { organization: 'test-org-from-config', apis: {} };
        process.env.REDOCLY_AUTHORIZATION = 'test-api-key';
        fsStatSyncSpy.mockReturnValueOnce({
            isDirectory() {
                return false;
            },
        });
        pathResolveSpy.mockImplementationOnce((p) => p);
        pathRelativeSpy.mockImplementationOnce((_, p) => p);
        pathDirnameSpy.mockImplementation((_) => '.');
        yield (0, push_1.handlePush)({
            domain: 'test-domain',
            'mount-path': 'test-mount-path',
            project: 'test-project',
            branch: 'test-branch',
            author: 'TestAuthor <test-author@mail.com>',
            message: 'Test message',
            files: ['test-file'],
            'default-branch': 'main',
            'max-execution-time': 10,
        }, mockConfig);
        expect(remotes.getDefaultBranch).toHaveBeenCalledWith('test-org-from-config', expect.anything());
        expect(remotes.upsert).toHaveBeenCalledWith('test-org-from-config', expect.anything(), expect.anything());
        expect(remotes.push).toHaveBeenCalledWith('test-org-from-config', expect.anything(), expect.anything(), expect.anything());
    }));
    it('should get domain from env if not passed', () => __awaiter(void 0, void 0, void 0, function* () {
        const mockConfig = { organization: 'test-org-from-config', apis: {} };
        process.env.REDOCLY_AUTHORIZATION = 'test-api-key';
        process.env.REDOCLY_DOMAIN = 'test-domain-from-env';
        fsStatSyncSpy.mockReturnValueOnce({
            isDirectory() {
                return false;
            },
        });
        pathResolveSpy.mockImplementationOnce((p) => p);
        pathRelativeSpy.mockImplementationOnce((_, p) => p);
        pathDirnameSpy.mockImplementation((_) => '.');
        yield (0, push_1.handlePush)({
            'mount-path': 'test-mount-path',
            project: 'test-project',
            branch: 'test-branch',
            'default-branch': 'main',
            author: 'TestAuthor <test-author@mail.com>',
            message: 'Test message',
            files: ['test-file'],
            'max-execution-time': 10,
        }, mockConfig);
        expect(api_1.ReuniteApiClient).toBeCalledWith('test-domain-from-env', 'test-api-key');
    }));
});
